@@ -53,8 +53,20 @@ namespace Implementation {
         template<class T> constexpr static T pow(T) { return T(1); }
     };
 
-    template<class> struct IsBoolVector: std::false_type {};
-    template<std::size_t size> struct IsBoolVector<BoolVector<size>>: std::true_type {};
+    template<class> struct IsBoolVectorOrScalar: std::false_type {};
+    template<> struct IsBoolVectorOrScalar<bool>: std::true_type {};
+    template<std::size_t size> struct IsBoolVectorOrScalar<BoolVector<size>>: std::true_type {};
+
+    template<class T> struct IsVectorOrScalar: std::is_arithmetic<T>::type {};
+    template<template<class> class Derived, class T> struct IsVectorOrScalar<Unit<Derived, T>>: std::true_type {};
+    template<class T> struct IsVectorOrScalar<Deg<T>>: std::true_type {};
+    template<class T> struct IsVectorOrScalar<Rad<T>>: std::true_type {};
+    template<std::size_t size, class T> struct IsVectorOrScalar<Vector<size, T>>: std::true_type {};
+    template<class T> struct IsVectorOrScalar<Vector2<T>>: std::true_type {};
+    template<class T> struct IsVectorOrScalar<Vector3<T>>: std::true_type {};
+    template<class T> struct IsVectorOrScalar<Vector4<T>>: std::true_type {};
+    template<class T> struct IsVectorOrScalar<Color3<T>>: std::true_type {};
+    template<class T> struct IsVectorOrScalar<Color4<T>>: std::true_type {};
 }
 
 /**
@@ -79,7 +91,7 @@ UnsignedInt MAGNUM_EXPORT log2(UnsignedInt number);
 Returns natural (base @f$ e @f$) logarithm of given number.
 @see @ref Constants::e(), @ref log(UnsignedInt, UnsignedInt), @ref log2()
 */
-template<class T> T log(T number) { return std::log(number); }
+template<class T> inline T log(T number) { return std::log(number); }
 
 /**
 @brief Natural exponential
@@ -87,7 +99,7 @@ template<class T> T log(T number) { return std::log(number); }
 Returns @f$ e^x @f$.
 @see @ref Constants::e(), @ref pow(T, T)
 */
-template<class T> T exp(T exponent) { return std::exp(exponent); }
+template<class T> inline T exp(T exponent) { return std::exp(exponent); }
 
 /**
 @brief Integer division with remainder
@@ -106,12 +118,47 @@ Int quotient = 57/6;
 Int remainder = 57%6;
 @endcode
 */
-template<class Integral> std::pair<Integral, Integral> div(Integral x, Integral y) {
+template<class Integral> inline std::pair<Integral, Integral> div(Integral x, Integral y) {
     static_assert(std::is_integral<Integral>{}, "Math::div(): not an integral type");
     const auto result = std::div(x, y);
     return {result.quot, result.rem};
 }
 
+/**
+@brief If given number is a positive or negative infinity
+
+@see @ref isNan(), @ref Constants::inf()
+*/
+template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, bool>::type isInf(T value) {
+    return std::isinf(value);
+}
+
+/** @overload */
+template<std::size_t size, class T> inline BoolVector<size> isInf(const Vector<size, T>& value) {
+    BoolVector<size> out;
+    for(std::size_t i = 0; i != size; ++i)
+        out.set(i, std::isinf(value[i]));
+    return out;
+}
+
+/**
+@brief If given number is a NaN
+
+Equivalent to @cpp value != value @ce.
+@see @ref isInf(), @ref Constants::nan()
+*/
+template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, bool>::type isNan(T value) {
+    return std::isnan(value);
+}
+
+/** @overload */
+template<std::size_t size, class T> inline BoolVector<size> isNan(const Vector<size, T>& value) {
+    BoolVector<size> out;
+    for(std::size_t i = 0; i != size; ++i)
+        out.set(i, std::isnan(value[i]));
+    return out;
+}
+
 /** @todo Can't trigonometric functions be done with only one overload? */
 
 /* The functions accept Unit instead of Rad to make them working with operator
@@ -193,7 +240,7 @@ template<UnsignedInt exponent, class T> constexpr T pow(T base);
 template<UnsignedInt exponent, class T> constexpr typename std::enable_if<std::is_arithmetic<T>::value, T>::type pow(T base) {
     return Implementation::Pow<exponent>::pow(base);
 }
-template<UnsignedInt exponent, std::size_t size, class T> Vector<size, T> pow(const Vector<size, T>& base) {
+template<UnsignedInt exponent, std::size_t size, class T> inline Vector<size, T> pow(const Vector<size, T>& base) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = Implementation::Pow<exponent>::pow(base[i]);
@@ -210,7 +257,7 @@ Returns power of @p base to the @p exponent.
 #ifdef DOXYGEN_GENERATING_OUTPUT
 template<class T> T pow(T base, T exponent);
 #else
-template<class T> typename std::enable_if<std::is_arithmetic<T>::value, T>::type pow(T base, T exponent) {
+template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type pow(T base, T exponent) {
     return std::pow(base, exponent);
 }
 template<std::size_t size, class T> inline Vector<size, T> pow(const Vector<size, T>& base, T exponent) {
@@ -254,7 +301,7 @@ template<std::size_t size, class T> inline Vector<size, T> min(const Vector<size
 
 If the range is empty, returns default-constructed value.
 */
-template<class T> T min(Corrade::Containers::ArrayView<const T> range) {
+template<class T> inline T min(Corrade::Containers::ArrayView<const T> range) {
     if(range.empty()) return {};
 
     T out(range[0]);
@@ -268,6 +315,11 @@ template<class T> inline T min(std::initializer_list<T> list) {
     return min(Corrade::Containers::ArrayView<const T>{list.begin(), list.size()});
 }
 
+/** @overload */
+template<class T, std::size_t size> inline T min(const T(&array)[size]) {
+    return min(Corrade::Containers::arrayView(array));
+}
+
 /**
 @brief Maximum
 
@@ -289,7 +341,7 @@ template<std::size_t size, class T> Vector<size, T> max(const Vector<size, T>& v
 #endif
 
 /** @overload */
-template<std::size_t size, class T> Vector<size, T> max(const Vector<size, T>& value, T max) {
+template<std::size_t size, class T> inline Vector<size, T> max(const Vector<size, T>& value, T max) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = std::max(value[i], max);
@@ -301,7 +353,7 @@ template<std::size_t size, class T> Vector<size, T> max(const Vector<size, T>& v
 
 If the range is empty, returns default-constructed value.
 */
-template<class T> T max(Corrade::Containers::ArrayView<const T> range) {
+template<class T> inline T max(Corrade::Containers::ArrayView<const T> range) {
     if(range.empty()) return {};
 
     T out(range[0]);
@@ -315,10 +367,16 @@ template<class T> inline T max(std::initializer_list<T> list) {
     return max(Corrade::Containers::ArrayView<const T>{list.begin(), list.size()});
 }
 
+/** @overload */
+template<class T, std::size_t size> inline T max(const T(&array)[size]) {
+    return max(Corrade::Containers::arrayView(array));
+}
+
 /**
 @brief Minimum and maximum of two values
 
-@see @ref min(), @ref max(), @ref clamp(), @ref Vector::minmax()
+@see @ref min(), @ref max(), @ref clamp(), @ref Vector::minmax(),
+    @ref Range::Range(const std::pair<VectorType, VectorType>&)
 */
 #ifdef DOXYGEN_GENERATING_OUTPUT
 template<class T> inline std::pair<T, T> minmax(const T& a, const T& b);
@@ -326,7 +384,7 @@ template<class T> inline std::pair<T, T> minmax(const T& a, const T& b);
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, std::pair<T, T>>::type minmax(T a, T b) {
     return a < b ? std::make_pair(a, b) : std::make_pair(b, a);
 }
-template<std::size_t size, class T> std::pair<Vector<size, T>, Vector<size, T>> minmax(const Vector<size, T>& a, const Vector<size, T>& b) {
+template<std::size_t size, class T> inline std::pair<Vector<size, T>, Vector<size, T>> minmax(const Vector<size, T>& a, const Vector<size, T>& b) {
     using std::swap;
     std::pair<Vector<size, T>, Vector<size, T>> out{a, b};
     for(std::size_t i = 0; i != size; ++i)
@@ -352,8 +410,9 @@ namespace Implementation {
 @brief Minimum and maximum of a range
 
 If the range is empty, returns default-constructed values.
+@see @ref Range::Range(const std::pair<VectorType, VectorType>&)
 */
-template<class T> std::pair<T, T> minmax(Corrade::Containers::ArrayView<const T> range) {
+template<class T> inline std::pair<T, T> minmax(Corrade::Containers::ArrayView<const T> range) {
     if(range.empty()) return {};
 
     T min{range[0]}, max{range[0]};
@@ -368,6 +427,11 @@ template<class T> inline std::pair<T, T> minmax(std::initializer_list<T> list) {
     return minmax(Corrade::Containers::ArrayView<const T>{list.begin(), list.size()});
 }
 
+/** @overload */
+template<class T, std::size_t size> inline std::pair<T, T> minmax(const T(&array)[size]) {
+    return minmax(Corrade::Containers::arrayView(array));
+}
+
 /**
 @brief Clamp value
 
@@ -387,7 +451,7 @@ template<class T, class U> inline T clamp(const T& value, const T& min, const T&
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type clamp(T value, T min, T max) {
     return std::min(std::max(value, min), max);
 }
-template<std::size_t size, class T> Vector<size, T> clamp(const Vector<size, T>& value, const Vector<size, T>& min, const Vector<size, T>& max) {
+template<std::size_t size, class T> inline Vector<size, T> clamp(const Vector<size, T>& value, const Vector<size, T>& min, const Vector<size, T>& max) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = clamp(value[i], min[i], max[i]);
@@ -396,7 +460,7 @@ template<std::size_t size, class T> Vector<size, T> clamp(const Vector<size, T>&
 #endif
 
 /** @overload */
-template<std::size_t size, class T> Vector<size, T> clamp(const Vector<size, T>& value, T min, T max) {
+template<std::size_t size, class T> inline Vector<size, T> clamp(const Vector<size, T>& value, T min, T max) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = clamp(value[i], min, max);
@@ -416,7 +480,7 @@ template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T
     if(scalar < T(0)) return T(-1);
     return T(0);
 }
-template<std::size_t size, class T> Vector<size, T> sign(const Vector<size, T>& a) {
+template<std::size_t size, class T> inline Vector<size, T> sign(const Vector<size, T>& a) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = sign(a[i]);
@@ -431,7 +495,7 @@ template<class T> inline T abs(const T& a);
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type abs(T a) {
     return std::abs(a);
 }
-template<std::size_t size, class T> Vector<size, T> abs(const Vector<size, T>& a) {
+template<std::size_t size, class T> inline Vector<size, T> abs(const Vector<size, T>& a) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = std::abs(a[i]);
@@ -446,7 +510,7 @@ template<class T> inline T floor(const T& a);
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type floor(T a) {
     return std::floor(a);
 }
-template<std::size_t size, class T> Vector<size, T> floor(const Vector<size, T>& a) {
+template<std::size_t size, class T> inline Vector<size, T> floor(const Vector<size, T>& a) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = std::floor(a[i]);
@@ -459,22 +523,12 @@ template<std::size_t size, class T> Vector<size, T> floor(const Vector<size, T>&
 template<class T> inline T round(const T& a);
 #else
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type round(T a) {
-    /** @todo Remove when newlib has this fixed */
-    #ifndef CORRADE_TARGET_ANDROID
     return std::round(a);
-    #else
-    return (a > T(0)) ? std::floor(a + T(0.5)) : std::ceil(a - T(0.5));
-    #endif
 }
-template<std::size_t size, class T> Vector<size, T> round(const Vector<size, T>& a) {
+template<std::size_t size, class T> inline Vector<size, T> round(const Vector<size, T>& a) {
     Vector<size, T> out{NoInit};
-    for(std::size_t i = 0; i != size; ++i) {
-        #ifndef CORRADE_TARGET_ANDROID
+    for(std::size_t i = 0; i != size; ++i)
         out[i] = std::round(a[i]);
-        #else
-        out[i] = round(a[i]);
-        #endif
-    }
     return out;
 }
 #endif
@@ -486,7 +540,7 @@ template<class T> inline T ceil(const T& a);
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type ceil(T a) {
     return std::ceil(a);
 }
-template<std::size_t size, class T> Vector<size, T> ceil(const Vector<size, T>& a) {
+template<std::size_t size, class T> inline Vector<size, T> ceil(const Vector<size, T>& a) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = std::ceil(a[i]);
@@ -505,7 +559,7 @@ template<class T> inline T sqrt(const T& a);
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type sqrt(T a) {
     return T(std::sqrt(a));
 }
-template<std::size_t size, class T> Vector<size, T> sqrt(const Vector<size, T>& a) {
+template<std::size_t size, class T> inline Vector<size, T> sqrt(const Vector<size, T>& a) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = T(std::sqrt(a[i]));
@@ -525,7 +579,7 @@ template<class T> inline T sqrtInverted(const T& a);
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type sqrtInverted(T a) {
     return T(1)/std::sqrt(a);
 }
-template<std::size_t size, class T> Vector<size, T> sqrtInverted(const Vector<size, T>& a) {
+template<std::size_t size, class T> inline Vector<size, T> sqrtInverted(const Vector<size, T>& a) {
     return Vector<size, T>(T(1))/sqrt(a);
 }
 #endif
@@ -537,21 +591,34 @@ template<std::size_t size, class T> Vector<size, T> sqrtInverted(const Vector<si
 @param t     Interpolation phase (from range @f$ [0; 1] @f$)
 
 The interpolation for vectors is done as in following, similarly for scalars: @f[
-    \boldsymbol v_{LERP} = (1 - t) \boldsymbol v_A + t \boldsymbol v_B
+    \boldsymbol{v_{LERP}} = (1 - t) \boldsymbol{v_A} + t \boldsymbol{v_B}
 @f]
-@see @ref lerpInverted(), @ref lerp(const Quaternion<T>&, const Quaternion<T>&, T)
+
+See @ref select() for constant interpolation using the same API and
+@ref splerp() for spline interpolation.
+@see @ref lerpInverted(), @ref lerp(const Complex<T>&, const Complex<T>&, T),
+    @ref lerp(const Quaternion<T>&, const Quaternion<T>&, T),
+    @ref lerp(const CubicHermite<T>&, const CubicHermite<T>&, U),
+    @ref lerp(const CubicHermiteComplex<T>&, const CubicHermiteComplex<T>&, T),
+    @ref lerp(const CubicHermiteQuaternion<T>&, const CubicHermiteQuaternion<T>&, T)
 @m_keyword{mix(),GLSL mix(),}
 */
-#ifdef DOXYGEN_GENERATING_OUTPUT
-template<class T, class U> inline T lerp(const T& a, const T& b, U t);
-#else
-template<class T, class U> inline typename std::enable_if<!Implementation::IsBoolVector<U>::value, T>::type lerp(T a, T b, U t) {
-    return T(Implementation::lerp(a, b, t));
-}
-template<std::size_t size, class T, class U> inline typename std::enable_if<!Implementation::IsBoolVector<U>::value, Vector<size, T>>::type lerp(const Vector<size, T>& a, const Vector<size, T>& b, U t) {
+template<class T, class U> inline
+    #ifndef DOXYGEN_GENERATING_OUTPUT
+    typename std::enable_if<Implementation::IsVectorOrScalar<T>::value && !Implementation::IsBoolVectorOrScalar<U>::value, T>::type
+    #else
+    T
+    #endif
+lerp(const T& a, const T& b, U t) {
     return Implementation::lerp(a, b, t);
 }
-#endif
+
+/** @overload
+@m_keyword{mix(),GLSL mix(),}
+*/
+template<class T> inline T lerp(const T& a, const T& b, bool t) {
+    return t ? b : a;
+}
 
 /** @overload
 Similar to the above, but instead of multiplication and addition it just does
@@ -584,9 +651,9 @@ template<std::size_t size> inline BoolVector<size> lerp(const BoolVector<size>&
 @param lerp Interpolated value
 
 Returns interpolation phase *t*: @f[
-    t = \frac{\boldsymbol v_{LERP} - \boldsymbol v_A}{\boldsymbol v_B - \boldsymbol v_A}
+    t = \frac{\boldsymbol{v_{LERP}} - \boldsymbol{v_A}}{\boldsymbol{v_B} - \boldsymbol{v_A}}
 @f]
-@see @ref lerp()
+@see @ref lerp(), @ref select()
 */
 #ifdef DOXYGEN_GENERATING_OUTPUT
 template<class T> inline T lerpInverted(const T& a, const T& b, const T& lerp);
@@ -599,6 +666,25 @@ template<std::size_t size, class T, class U> inline Vector<size, T> lerpInverted
 }
 #endif
 
+/**
+@brief Constant interpolation of two values
+@param a     First value
+@param b     Second value
+@param t     Interpolation phase
+
+A constant interpolation counterpart to @ref lerp(): @f[
+    \boldsymbol{v}_i = \begin{cases}
+        \boldsymbol{v_A}_i, & t_i < 1 \\
+        \boldsymbol{v_B}_i, & t_i \ge 1
+    \end{cases}
+@f]
+
+Equivalent to calling @cpp Math::lerp(a, b, t >= U(1)) @ce.
+*/
+template<class T, class U> constexpr T select(const T& a, const T& b, U t) {
+    return lerp(a, b, t >= U(1));
+}
+
 /**
 @brief Fused multiply-add
 
@@ -609,11 +695,9 @@ doing the computation manually.
 template<class T> inline T fma(const T& a, const T& b, const T& c);
 #else
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type fma(T a, T b, T c) {
-    /** @todo Remove when newlib has this fixed */
     /* On Emscripten it works with -O2 but not with -O1 (function not defined).
-       I guess that's only because -O2 optimizes it out, so disabling it there
-       also */
-    #if !defined(CORRADE_TARGET_ANDROID) && !defined(CORRADE_TARGET_EMSCRIPTEN)
+       I guess that's only because -O2 optimizes it out, so disabling it there. */
+    #ifndef CORRADE_TARGET_EMSCRIPTEN
     return std::fma(a, b, c);
     #else
     return a*b + c;
